(domain (name ""))

(object-class Root nil)

(object-class Meeting (Root)
  (actions :syntax list)
  (items :syntax list)
  (items-done :syntax list)
  (time :syntax number)
  (date :syntax number)
  (status :syntax string)
  (participants :syntax list)
  (minutes :syntax string)
  (end-time :syntax number))

(object-class Avatar (Person)
  (longitude :syntax number)
  (latitude :syntax number)
  (elevation :syntax number))

(object-class Action (Root)
  (person :syntax object)
  (description :syntax string)
  (item :syntax object))

(object-class Item (Root)
  (papers :syntax list)
  (meeting :syntax object)
  (description :syntax string)
  (addressed :syntax symbol)
  (name :syntax string)
  (images :syntax list))

(object-class Person (Root)
  (name :syntax string)
  (role :syntax string)
  (present :syntax symbol)
  (apologises :syntax symbol))

(object-class Human (Person))

(object-class Project (Root)
  (meeting-number :syntax number)
  (milestones :syntax list)
  (name :syntax string))

(object-class Milestone (Root)
  (date :syntax number)
  (description :syntax string))

(object-class Decision (Root)
  (description :syntax string)
  (item :syntax object))

(refinement "2-finish-meeting" (finish-meeting ?meeting)
  (nodes
    (1 (date-next-meeting ?meeting))
    (2 (save-meeting-notes ?meeting)))
  (orderings
    (1 2)))

(refinement "1-hold-meeting" (hold-meeting ?meeting)
  (nodes
    (1 (collect-participants ?meeting))
    (2 (start-meeting ?meeting))
    (3 (address-previous-actions ?meeting))
    (4 (address-items ?meeting))
    (5 (finish-meeting ?meeting)))
  (orderings
    (1 2) (2 3) (2 4) (3 5) (4 5)))

(refinement "2-wait-for-participants" (collect-participants ?meeting)
  (constraints
    (world-state condition (participants ?meeting) = ?people)
    (world-state condition (status ?meeting) = waiting)
    (world-state effect (status ?meeting) = ready)
    (compute  ((lambda (people) (let ((is-ok TRUE)) (for-each (lambda (person) (if (or (eq (get-world-state-value (list (quote present) person)) (quote true)) (eq (get-world-state-value (list (quote apologises) person)) (quote true))) nil (setq is-ok FALSE))) people) is-ok)) ?people) = true)))

(refinement "0-13-add-person" (add-person ?meeting ?type ?person)
  (constraints
    (world-state condition (participants ?meeting) = ?participants)
    (world-state effect (name ?person) = ?person)
    (world-state effect (participants ?meeting) = ?newParts)
    (world-state effect (present ?person) = false)
    (world-state effect (type ?person) = ?type)
    (compute  (cons ?person ?participants) = ?newParts)))

(refinement "0-10-init-meeting" (init-meeting ?meeting)
  (constraints
    (world-state effect (actions ?meeting) = nil)
    (world-state effect (items ?meeting) = nil)
    (world-state effect (participants ?meeting) = nil)
    (world-state effect (status ?meeting) = waiting)
    (world-state effect (type ?meeting) = Meeting)))

(refinement "3-present-ref-item" (present-item ?meeting ?item)
  (nodes
    (1 (image ?ref)))
  (constraints
    (world-state condition (ref ?item) = ?ref)
    (world-state condition (status ?meeting) = ready)
    (world-state condition (type ?item) = Item)
    (world-state condition (type ?meeting) = Meeting)))

(refinement "2-address-items" (address-items ?meeting)
  (nodes
    (1 (select-item ?meeting ?item))
    (2 (present-item ?meeting ?item))
    (3 (discuss-item ?item))
    (4 (note-actions ?meeting)))
  (orderings
    (1 2) (1 4)))

(refinement "0-11add-item" (add-item ?meeting ?item ?itemText)
  (constraints
    (world-state condition (items ?meeting) = ?items)
    (world-state condition (type ?meeting) = Meeting)
    (world-state effect (addressed ?item) = false)
    (world-state effect (description ?item) = ?itemText)
    (world-state effect (items ?meeting) = ?itemsNew)
    (world-state effect (type ?item) = Item)
    (compute (cons ?item ?items) = ?itemsNew)))

(refinement "0-1-prepare-demo-meeting" (prepare-meeting ?meeting)
  (nodes
    (1 (init-meeting ?meeting))
    (2 (add-person ?meeting Avatar "Jussi Aya"))
    (3 (add-person ?meeting Avatar "SP Pizzicato"))
    (4 (add-item ?meeting octopus1 "the good octopus" A))
    (5 (add-item ?meeting octopus2 "the naff octopus" B)))
  (orderings
    (1 2) (1 3)))

(refinement "0-12-add-named-item" (add-item ?meeting ?item ?itemText ?itemRef)
  (nodes
    (1 (add-item ?meeting ?item ?itemText)))
  (constraints
    (world-state condition (type ?meeting) = Meeting)
    (world-state effect (ref ?item) = ?itemRef)))

(refinement "0-0-run" (run)
  (nodes
    (1 (prepare-meeting iRoom))
    (2 (hold-meeting iRoom))))

(refinement "2-start-meeting" (start-meeting ?meeting)
  (nodes
    (1 (welcome-participants ?meeting))
    (2 (note apologies ?meeting))
    (3 (agree-end ?meeting))
    (4 (agree-previous-minutes ?meeting)))
  (orderings
    (1 2) (1 3) (2 4) (3 4)))

(refinement add-apology (add-apology ?meeting ?person)
  (constraints
    (world-state condition (type ?meeting) = Meeting)
    (world-state condition (type ?person) = Person)
    (world-state effect (apologises ?person) = true)))

(refinement "0-load-meeting" (prepare-meeting ?meeting ?meeting-file))

(refinement "3-agree-previous-minutes ?meeting" (agree-previous-minutes ?meeting)
  (nodes
    (2 (agree-minutes ?minutes)))
  (constraints
    (world-state condition (minutes ?prev-meet) = ?minutes)
    (world-state condition (previous-meeting ?meeting) = ?prev-meet)
    (world-state condition (type ?meeting) = Meeting)
    (world-state condition (type ?prev-meet) = Meeting)))

(refinement add-person-present (add-apology ?meeting ?person)
  (constraints
    (world-state condition (type ?meeting) = Meeting)
    (world-state condition (type ?person) = Person)
    (world-state effect (apologises ?person) = false)
    (world-state effect (present ?person) = true)))

(refinement "0-setup-project" (setup-ptoject ?project)
  (constraints
    (world-state effect (meeting-number ?project) = 0)
    (world-state effect (name ?project) = ?project)
    (world-state effect (type ?project) = Project)))

(refinement "0-setup-next-project-meeting" (setup-next-project-meeting ?project ?date)
  (constraints
    (world-state condition (meeting-number ?project) = ?meet-number)
    (world-state condition (type ?project) = Project)))

(annotations
  (comments = "Objects for meeting-related things (people, actions, items, etc.)\
SOPs for setting up meetings (all marked 0-...)\
SOPs for holding meetings."))
