/* Author: Jeff Dalton <J.Dalton@ed.ac.uk>
 * Updated: Wed Feb 27 17:20:05 2008 by Jeff Dalton
 * Copyright: (c) 2008, AIAI, University of Edinburgh
 */

package ix.imeet;

import java.util.*;

import ix.icore.*;
import ix.icore.domain.*;
import ix.icore.log.HistoryEvent;

import ix.ip2.*;

import ix.imeet.meeting.Meeting;

import ix.util.*;
import ix.util.lisp.*;
import ix.util.match.*;
import ix.util.xml.XML;

/**
 * Write a description of the next meeting.
 */
public class SetupNextMeetingHandler extends ActivityHandler {

    private final LList handleableActionPattern =
	Lisp.elementsFromString("setup-next-meeting");

    private final LList syntaxList = Lisp.list(handleableActionPattern);

    private Ip2 ip2;

    public SetupNextMeetingHandler(Ip2 ip2) {
        super("Set up for the next meeting");
        this.ip2 = ip2;
    }

    public List getSyntaxList() {
	return syntaxList;
    }

    public boolean appliesTo(AgendaItem item) {
	return SimpleMatcher.matchAnyPattern(syntaxList, item.getPattern())
	    != null;
    }

    public void addHandlerActions(AgendaItem item) {
        item.addAction
            (new HandlerAction.AutomaticWhenBound(item, this));
    }

    public void handle(AgendaItem item) {

	// Find out what happened during the meeting.
	IMeetEventLogger logger =
	    Util.mustBe(IMeetEventLogger.class, ip2.getEventLogger());
	List<HistoryEvent> historyEvents = logger.getEvents();

	// Write minutes.
	MinutesMaker mm = new MinutesMaker(ip2, historyEvents);
	Meeting meeting = mm.makeMeetingDescription();
	MinutesHtmlOutputter out =
	    new MinutesHtmlOutputter("tmp/minutes.html");
	out.writeMinutes(meeting);

	// Write next-meeting domain.
	NextMeetingPrep prep = new NextMeetingPrep(ip2, meeting);
	Domain dom = prep.makeNextMeetingAgendaDomain();
	XML.writeObject(dom, "tmp/next-meeting-agenda.lsp");

	// The acvtivity is now complete.
	item.setStatus(Status.COMPLETE);

    }

}
